/*Class name: SendAlert
 *Namespace: DVM_Management_Center.Mail
 *Author: Harald Gsoedl
 *Last change: Benno Rott
 *Changed: 21/02/2007
 *Use: Provides routines for sending an alert-email
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

using DVM_Management_Center.FileIO;
using DVM_Management_Center.Objects;

namespace DVM_Management_Center.Mail
{
    /// <summary>
    /// Provides routines for sending an alert-email
    /// </summary>
    class SendAlert:SendMail
    {
        private FileIO.FileIO file;
        private String[] sentStatus;
        private int[] sent;

        private AlertSettings _alertSettings;
        public AlertSettings AlertSettings
        {
            get { return _alertSettings; }
            set { 
                _alertSettings = value; 
            }
        }

        private DateTime[] _sentTime;
        public DateTime[] SentTime
        {
            get { return _sentTime; }
            set { _sentTime = value; }
        }

        private int[] _lastMessage;
        public int[] LastMessage
        {
            get { return _lastMessage; }
            set { _lastMessage = value; }
        }

        private int[] _yelCount;
        public int[] YelCount
        {
            get { return _yelCount; }
            set { _yelCount = value; }
        }
        private int[] _redCount;
        public int[] RedCount
        {
            get { return _redCount; }
            set { _redCount = value; }
        }
        private int[] _synCount;
        public int[] SynCount
        {
            get { return _synCount; }
            set { _synCount = value; }
        }
        private int[] _conCount;
        public int[] ConCount
        {
            get { return _conCount; }
            set { _conCount = value; }
        }

        private DVMObject[] _dvmObjects;
        public DVMObject[] DVMObjects
        {
            get { return _dvmObjects; }
            set { _dvmObjects = value; }
        }

        private Hashtable _tempLogs;
        public Hashtable TempLogs
        {
            get { return _tempLogs; }
            set { _tempLogs = value; }
        }

        private String _lastLog;
        public String LastLog
        {
            get { return _lastLog; }
            set { _lastLog = value; }
        }

        /// <summary>
        /// Class for sending the email-alert
        /// </summary>
        /// <param name="count"></param>
        public SendAlert(int count, AlertSettings alertSettings):base(alertSettings.Host, alertSettings.Port, alertSettings.Username, alertSettings.Password)
        {
            file      = new FileIO.FileIO();
            _alertSettings = alertSettings;
            _synCount = new int[count];
            _yelCount = new int[count];
            _conCount = new int[count];
            _redCount = new int[count];
            _dvmObjects = new DVMObject[count];
            _sentTime = new DateTime[count];
            _tempLogs = new Hashtable();
            sent = new int[count];
            _lastMessage = new int[count];

            sentStatus = new String[5];
            sentStatus[0] = "No alarm";
            sentStatus[1] = "Entry in alarm report";
            sentStatus[2] = "Continuous Alarm Status";
            sentStatus[3] = "TS Sync loss";
            sentStatus[4] = "Connectivity loss";

            _lastLog = file.ReadLastLog();
            for(int i = 1; i<sentStatus.Length; i++)
            {
                if(_lastLog.Contains(sentStatus[i]))
                {
                    _lastLog = i.ToString() + _lastLog;
                }
            }
        }

        /// <summary>
        /// Set everything ok, when the DVM is ok
        /// </summary>
        /// <param name="i"></param>
        public void SetOK(int i)
        {
            _synCount[i] = 0;
            _redCount[i]  = 0;
            _yelCount[i] = 0;
            _conCount[i] = 0;
            sent[i]      = 0;
        }

        /// <summary>
        /// This methode includes the logic to send the alert
        /// </summary>
        public void SendTheAlert()
        {
            try
            {
                String error = String.Empty;
                for (int i = 0; i < _synCount.Length; i++) //Go through every DVM
                {
                    if (sent[i] > 0 && _sentTime[i].AddHours(_alertSettings.HoursDelay).Ticks < DateTime.Now.Ticks) // Delay time
                        sent[i] = 0;

                    // Fixes for int32-overflows
                    if (_conCount[i] < 0) _conCount[i] = 0;
                    if (_synCount[i] < 0) _synCount[i] = 0;
                    if (_redCount[i] < 0) _redCount[i] = 0;
                    if (_yelCount[i] < 0) _yelCount[i] = 0;

                    int current_problem = 0;

                    if (_alertSettings.BlEntry
                        && (_lastMessage[i] == 1
                            || _lastMessage[i] == 2 && !_alertSettings.BlAlarm
                            || _lastMessage[i] == 3 && !_alertSettings.BlSyncLoss
                            )
                        && sent[i] < 1
                        )
                    {
                        current_problem = 1; // Entry in alarm report
                    }
                    if (_lastMessage[i] == 2
                        && _alertSettings.BlAlarm
                        && _redCount[i] >= _alertSettings.TbAlarm
                        && sent[i] < 2
                        )
                    {
                        current_problem = 2; // Continuous alarm status
                    }
                    if (_lastMessage[i] == 3
                        && _alertSettings.BlSyncLoss
                        && _synCount[i] >= _alertSettings.TbSyncLoss
                        && sent[i] < 3
                        )
                    {
                        current_problem = 3; // TS Sync loss
                    }
                    if (_lastMessage[i] == 4
                        && _alertSettings.BlConnectivity
                        && _conCount[i] >= _alertSettings.TbConnectivity
                        && sent[i] < 4
                        )
                    {
                        current_problem = 4; // Connectivity loss
                    }

                    if (current_problem != 0)
                    {
                        String text;
                        text = "IP: " + _dvmObjects[i].Ip + "\r\n"
                             + "Site Name: " + _dvmObjects[i].SiteName + "\r\n"
                             + "Type of error: " + sentStatus[current_problem];

                        _sentTime[i] = DateTime.Now; // save for delay time
                        sent[i] = current_problem;

                        try
                        {
                            // Write the log-file
                            String logText = DateTime.Now.ToString() + ": " + _dvmObjects[i].Ip + " - " + sentStatus[current_problem];
                            file.WriteLogFile(logText);
                            _lastLog = current_problem.ToString() + logText;
                            _tempLogs.Add(_tempLogs.Count, logText);
                        }
                        catch (Exception ex)
                        {
                            error = ex.Message;
                        }

                        try
                        {
                            //Send the alert
                            this.SendAlert(_alertSettings.Receiver, _alertSettings.Sender, "DVM mail alert " + _dvmObjects[i].Ip, text);
                        }
                        catch (Exception ex)
                        {
                            throw new Exception(ex.Message);
                        }
                    }
                }
                if (error != String.Empty)
                {
                    throw new Exception(error);
                }
            }
            catch (Exception ex)
            {
               throw new Exception(ex.Message);
            }
        }
    }
}
